@@ -8,6 +8,7 @@ import time
 import asyncio
 import threading
 import numpy as np
+import queue
 from functools import wraps
 from typing import Callable, Coroutine
 from threading import RLock
@@ -16,29 +17,51 @@ _LOGGER = logging.getLogger(__name__)
 '''
 #ZxxPWRppp,SRCs,GRPt,VOL-yy
 '''
-CONSR_PATTERN = re.compile('Z(?P<zone>\d{2})'
+CONSR_PATTERN = re.compile('Z(?P<zone>\\d{2})'
                      'PWR(?P<power>ON|OFF),'
-                     'SRC(?P<source>\d),'
+                     'SRC(?P<source>\\d),'
                      'GRP(?P<group>0|1),'
-                     'VOL(?P<volume>-\d\d|-MT|-XM)')
+                     'VOL(?P<volume>-\\d\\d|-MT|-XM)')
+'''
+#ZzzPWRonoff,SRCs,VOLvvv
+'''
+STATUS_PATTERN = re.compile('Z(?P<zone>\\d{2})'
+                     'PWR(?P<power>ON|OFF),'
+                     'SRC(?P<source>\\d),'
+                     'VOL(?P<volume>-\\d\\d|-MT|-XM)')
+'''
+#ZzzPWROFF
+'''
+SIMPLESE_OFF_PATTERN = re.compile('Z(?P<zone>\\d{2})'
+                     'PWR(?P<power>ON|OFF)')
 '''
 #ZxxORp,BASSyy,TREByy,GRPq,VRSTr
 '''
-ESSENTIA_D_SETSR_PATTERN = re.compile('Z(?P<zone>\d{2})'
-                     'OR(?P<override>\d),'
-                     'BASS(?P<bass>[+-]?\d{2}),'
-                     'TREB(?P<treble>[+-]?\d{2}),'
+ESSENTIA_D_SETSR_PATTERN = re.compile('Z(?P<zone>\\d{2})'
+                     'OR(?P<override>\\d),'
+                     'BASS(?P<bass>[+-]?\\d{2}),'
+                     'TREB(?P<treble>[+-]?\\d{2}),'
                      'GRP(?P<group>0|1),'
                      'VRST(?P<volume_reset>1|0)')
 '''
 #Z0x,BASSyy,TREByy,GRPq
 '''
-SIMPLESE_SETSR_PATTERN = re.compile('Z(?P<zone>\d{2})'
+SIMPLESE_SETSR_PATTERN = re.compile('Z(?P<zone>\\d{2})'
                      '(?P<override>.)'
-                     'BASS(?P<bass>[+-]?\d{2}),'
-                     'TREB(?P<treble>[+-]?\d{2}),'
+                     'BASS(?P<bass>[+-]?\\d{2}),'
+                     'TREB(?P<treble>[+-]?\\d{2}),'
                      'GRP(?P<group>0|1)')
 '''
+#ZzzBASSequ,TREBequ,BALbal,Gg,MAXVOLmax,INIVOLini
+'''
+CONCERTO_SETSR_PATTERN = re.compile('Z(?P<zone>\\d{2})'
+                     'BASS(?P<bass>[+-]?\\d{2}),'
+                     'TREB(?P<treble>[+-]?\\d{2}),'
+                     'BAL(?P<balance>C|L+\\d|R+\\d),'
+                     'G(?P<group>\\d),'
+                     'MAXVOL(?P<maxvol>-\\d\\d),'
+                     'INIVOL(?P<inivol>-\\d\\d)')
+'''
 #MPU_E6D_vx.yy
 '''
 ESSENTIA_D_VERSION = re.compile('MPU_E6D[A-Za-z\t .]+')
@@ -47,42 +70,52 @@ ESSENTIA_D_VERSION = re.compile('MPU_E6D[A-Za-z\t .]+')
 '''
 SIMPLESE_VERSION = re.compile('MPU_A4D[A-Za-z\t .]+')
 '''
+#MPU-I8_FWvx.yy
+'''
+CONCERTO_VERSION = re.compile('MPU-I8_[A-Za-z\t .]+')
+'''
+#NV-I8G FWvx.yy HWvx
+'''
+CONCERTO_VERSION2 = re.compile('NV-I8G[A-Za-z\t .]+')
+'''
 #ALLOFF
 '''
 ALL_OFF_PATTERN = re.compile('ALLOFF')
 '''
 #ZxxLKlll
 '''
-KEYPAD_LOCK_PATTERN = re.compile('Z(?P<zone>\d{2})'
+KEYPAD_LOCK_PATTERN = re.compile('Z(?P<zone>\\d{2})'
                      'LK(?P<lock>ON|OFF)')
 '''
 #?
 '''
-ERROR_PATTERN = re.compile('\?')
+ERROR_PATTERN = re.compile('\\?')
 
 
 EOL = b'\r'
 TIMEOUT_OP       = 0.2   # Number of seconds before serial operation timeout
 TIMEOUT_RESPONSE = 1     # Number of seconds before command response timeout
+cmd_queue = queue.Queue()
 page_active = np.array([0,0])
-zoneinit = np.array([1,0,0,0,0,0,0,0,0,0,0,0,0])
-zonesetinit = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0])
-zonepwr = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0])
-zonepwrsave = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0])
-zonepwralloff = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0])
-zonevol = np.array([-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78])
-zonevolsave = np.array([-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78])
-zonesrc = np.array([1,0,0,0,0,0,0,0,0,0,0,0,0])
-zonesrcsave = np.array([0,1,1,1,1,1,1,1,1,1,1,1,1])
-zonemute = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0])
-zonemutesave = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0])
-zonelock = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0])
-zonebass = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0])
-zonetreble = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0])
-zonegroupsrc = np.array([1,1,1,1,1,1,1,1,1,1,1,1,1])
-zoneoverride = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0])
-zonevolreset = np.array([0,1,1,1,1,1,1,1,1,1,1,1,1])
-zonevoloffset = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0])
+zoneinit = np.array([1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
+zonesetinit = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
+zonepwr = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
+zonepwrsave = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
+zonepwralloff = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
+zonevol = np.array([-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78])
+zonevolsave = np.array([-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78,-78])
+zonesrc = np.array([1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
+zonesrcsave = np.array([0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1])
+zonemute = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
+zonemutesave = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
+zonelock = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
+zonebass = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
+zonetreble = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
+zonebalance = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
+zonegroupsrc = np.array([1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1])
+zoneoverride = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
+zonevolreset = np.array([0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1])
+zonevoloffset = np.array([0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
 setup = False
 
 class ZoneStatus(object):
@@ -90,7 +123,59 @@ class ZoneStatus(object):
                  ,zone: int
                  ,power: str
                  ,source: int
-                 ,group: bool
+                 ,volume: float  # -78 -> 0
+                 ,from_port: bool
+                 ):
+        self.source = str(source)
+        self.zone = abs(int(zone))
+        self.zonegroup_members = []
+        try:
+            zoneindexobj = [i for i, x in enumerate(media_zonetable) if x == zone]
+            zoneindex = int(zoneindexobj[0])
+            for grpzonembr in groupzone_members[zoneindex]:
+                entityindexobj = [i for i, x in enumerate(media_zonetable) if x == grpzonembr]
+                entityindex = int(entityindexobj[0])
+                self.zonegroup_members.append(entitytable[entityindex])
+        except:
+            self.zonegroup_members = None
+
+        if 'ON' in power:
+           self.power = bool(1)
+        else:
+           self.power = bool(0)
+
+        if 'MT' in volume:
+           self.mute = bool(1)
+           self.volume = zonevol[self.zone]
+        else:
+           self.mute = bool(0)
+           self.volume = int(volume)
+
+        zonepwr[self.zone] = self.power
+        zonevol[self.zone] = self.volume
+        zonesrc[self.zone] = self.source
+        zonemute[self.zone] = self.mute
+
+        _LOGGER.debug('Zone: %s, Power: %s, Vol: %sdb, %s%%, Mute: %s, Source: %s, Init: %s', \
+           self.zone, self.power, self.volume, db_to_percent(self.volume), \
+           self.mute, self.source, zoneinit[int(self.zone)])
+
+        if from_port:
+            if self.zone in media_zonetable:
+                callbackobj = media_zonetable.index(self.zone)
+                callbackint = int(callbackobj)
+                media_callback[callbackint]()
+
+    @classmethod
+    def from_string(cls, match):
+        ZoneStatus(*[str(m) for m in match.groups()], True)
+
+class ZoneConnectStatus(object):
+    def __init__(self
+                 ,zone: int
+                 ,power: str
+                 ,source: int
+                 ,group: int
                  ,volume: float  # -78 -> 0
                  ,from_port: bool
                  ):
@@ -116,11 +201,10 @@ class ZoneStatus(object):
                    if zonegroupsrc[count] == 0 and zonepwr[count] == 1:
                        if int(zonesrc[count]) != int(self.source):
                            zonesrc[count] = self.source
-                           if from_port:
-                               if zone in media_zonetable:
-                                   callbackobj = media_zonetable.index(count)
-                                   callbackint = int(callbackobj)
-                                   media_callback[callbackint]()
+                           if count in media_zonetable:
+                               callbackobj = media_zonetable.index(count)
+                               callbackint = int(callbackobj)
+                               media_callback[callbackint]()
 
         else:
            self.power = bool(0)
@@ -149,7 +233,7 @@ class ZoneStatus(object):
 
     @classmethod
     def from_string(cls, match):
-        ZoneStatus(*[str(m) for m in match.groups()], True)
+        ZoneConnectStatus(*[str(m) for m in match.groups()], True)
 
 class ZonesetStatus(object):
     def __init__(self
@@ -157,35 +241,38 @@ class ZonesetStatus(object):
              ,ovride: str
              ,bass: float
              ,treble: float
-             ,grp: int
+             ,group: int
              ,vrset: int
              ,from_port: bool
              ):
 
         self.zone = abs(int(zone))
-        self.treble = treble
         self.bass = bass
+        self.treble = treble
         zonebass[self.zone] = self.bass
         zonetreble[self.zone] = self.treble
+        zonegroupsrc[self.zone] = group
         self.volume_offset = zonevoloffset[self.zone]
+        zonevolreset[self.zone] = vrset
 
         if MODEL == 'ESSENTIA_D':
             zoneoverride[self.zone] = int(ovride)
-        zonevolreset[self.zone] = vrset
-        zonegroupsrc[self.zone] = grp
 
         if zoneoverride[self.zone] == 1:
            self.override = bool(1)
         else:
            self.override = bool(0)
+
         if zonevolreset[self.zone] == 1:
            self.volume_reset = bool(0)
         else:
            self.volume_reset = bool(1)
+
         if zonegroupsrc[self.zone] == 1:
            self.group = bool(0)
         else:
            self.group = bool(1)
+
         if zonelock[self.zone] == 0:
            self.keypad_lock = bool(0)
         else:
@@ -209,6 +296,52 @@ class ZonesetStatus(object):
         else:
             ZonesetStatus(*[str(m) for m in match.groups()], True)
 
+class ConcertoZonesetStatus(object):
+    def __init__(self
+             ,zone: int
+             ,bass: float
+             ,treble: float
+             ,balance: str
+             ,group: int
+             ,maxvol: float
+             ,inivol: float
+             ,from_port: bool
+             ):
+
+        self.zone = abs(int(zone))
+        self.bass = bass
+        self.treble = treble
+        zonebass[self.zone] = self.bass
+        zonetreble[self.zone] = self.treble
+        strbalance = str(balance)
+        if strbalance[0] == 'L':
+            self.balance = -int(strbalance[1])
+            zonebalance[self.zone] = self.balance
+        elif strbalance[0] == 'R':
+            self.balance = int(strbalance[1])
+            zonebalance[self.zone] = self.balance
+        elif strbalance[0] == 'C':
+            self.balance = 0
+            zonebalance[self.zone] = self.balance
+        else:    
+            self.balance = balance
+        self.volume_offset = zonevoloffset[self.zone]
+
+        if from_port:
+            if 'settings_zonetable' in globals():
+                if self.zone in settings_zonetable:
+                    callbackindex = [i for i, x in enumerate(settings_zonetable) if x == self.zone]
+                    for callbackzone in callbackindex:
+                        settings_callback[callbackzone]()
+
+        _LOGGER.debug('Zone: %s, Bass: %s, Treble: %s, Balance: %s, Init: %s',\
+            self.zone, self.bass, self.treble, self.balance, \
+            zonesetinit[int(self.zone)])
+
+    @classmethod
+    def from_string(cls, match):
+        ConcertoZonesetStatus(*[str(m) for m in match.groups()], True)
+
 class Nuvo(object):
     """
     Nuvo amplifier interface
@@ -228,7 +361,6 @@ class Nuvo(object):
     def zone_status(self, zone: int):
         """
         Get the structure representing the status of the zone
-        :param zone: zone 1.12
         :return: status of the zone or None
         """
         raise NotImplemented()
@@ -236,7 +368,6 @@ class Nuvo(object):
     def zoneset_status(self, zone: int):
         """
         Get the structure representing the status of the zone
-        :param zone: zone 1.12
         :return: status of the zone or None
         """
         raise NotImplemented()
@@ -244,7 +375,6 @@ class Nuvo(object):
     def set_power(self, zone: int, power: bool):
         """
         Turn zone on or off
-        :param zone: zone 1.12
         :param power: True to turn on, False to turn off
         """
         raise NotImplemented()
@@ -252,7 +382,6 @@ class Nuvo(object):
     def join_players(self, zone, group_members):
         """
         Join entity to a speaker group
-        :param zone: zone 1.12
         :param group_members: entity to add
         """
         raise NotImplemented()
@@ -260,14 +389,12 @@ class Nuvo(object):
     def unjoin_player(self, zone):
         """
         Removes zone from all speaker groups
-        :param zone to remove
         """
         raise NotImplemented()
 
     def set_mute(self, zone: int, mute: bool):
         """
         Mute zone on or off
-        :param zone: zone 1.12
         :param mute: True to mute, False to unmute
         """
         raise NotImplemented()
@@ -275,7 +402,6 @@ class Nuvo(object):
     def set_keypad_lock(self, zone: int, lock: bool):
         """
         Lock or unlock zone keypad
-        :param zone: zone 1.12
         :param mute: True to lock, False to unlock
         """
         raise NotImplemented()
@@ -283,7 +409,6 @@ class Nuvo(object):
     def set_volume(self, zone: int, volume: float):
         """
         Set volume for zone
-        :param zone: zone 1.12
         :param volume: float from -78 to 0 inclusive
         """
         raise NotImplemented()
@@ -291,7 +416,6 @@ class Nuvo(object):
     def set_volume_offset(self, zone: int, volume: float):
         """
         Set volume offset for groups
-        :param zone: zone 1.12
         :param volume: float from -30 to 30 inclusive
         """
         raise NotImplemented()
@@ -299,7 +423,6 @@ class Nuvo(object):
     def set_treble(self, zone: int, treble: float):
         """
         Set treble for zone
-        :param zone: zone 1.12
         :param treble: float from -12 to 12 inclusive
         """
         raise NotImplemented()
@@ -307,15 +430,20 @@ class Nuvo(object):
     def set_bass(self, zone: int, bass: float):
         """
         Set bass for zone
-        :param zone: zone 1.12
         :param bass: float from -12 to 12 inclusive
         """
         raise NotImplemented()
 
+    def set_balance(self, zone: int, balance: float):
+        """
+        Set balance for zone
+        :param balance: float from -6 to 6 inclusive
+        """
+        raise NotImplemented()
+
     def set_source(self, zone: int, source: int):
         """
         Set source for zone
-        :param zone: zone 1.12
         :param source: integer from 1 to 6 inclusive
         """
         raise NotImplemented()
@@ -385,6 +513,12 @@ def _parse_response(string: bytes):
        match = re.search(CONSR_PATTERN, string)
        if match:
           _LOGGER.debug('CONSR received')
+          ZoneConnectStatus.from_string(match)
+
+   if not match:
+       match = re.search(STATUS_PATTERN, string)
+       if match:
+          _LOGGER.debug('STATUS received')
           ZoneStatus.from_string(match)
 
    if not match:
@@ -393,6 +527,18 @@ def _parse_response(string: bytes):
           _LOGGER.debug('Simplese SETSR received')
           ZonesetStatus.from_string(match)
 
+   if not match:
+       match = re.search(CONCERTO_SETSR_PATTERN, string)
+       if match:
+          _LOGGER.debug('Concerto SETSR received')
+          ConcertoZonesetStatus.from_string(match)
+
+   if not match:
+       match = re.search(SIMPLESE_OFF_PATTERN, string)
+       if match:
+          _LOGGER.debug('Simplese PWROFF received')
+          SimpleseZonePwrOff(match)
+
    if not match:
        match = re.search(ALL_OFF_PATTERN, string)
        if match:
@@ -424,6 +570,18 @@ def _parse_response(string: bytes):
           _LOGGER.info('Nuvo returned model Simplese')
           MODEL = 'SIMPLESE'
 
+   if not match:
+       match = re.search(CONCERTO_VERSION, string)
+       if match:
+          _LOGGER.info('Nuvo returned model Concerto')
+          MODEL = 'CONCERTO'
+
+   if not match:
+       match = re.search(CONCERTO_VERSION2, string)
+       if match:
+          _LOGGER.info('Nuvo returned model Concerto (NV-I8G)')
+          MODEL = 'CONCERTO'
+
    if (string == '#Busy'):
        _LOGGER.warn('BUSY RESPONSE - TRY AGAIN')
    return None
@@ -436,7 +594,10 @@ def _format_version_request():
     return 'VER'.format()
 
 def _format_zone_status_request(zone: int) -> str:
-    return 'Z{:0=2}CONSR'.format(zone)
+    if MODEL == 'CONCERTO':
+        return 'Z{:0=2}STATUS'.format(zone)
+    else:
+        return 'Z{:0=2}CONSR'.format(zone)
 
 def _format_zoneset_status_request(zone: int) -> str:
     return 'Z{:0=2}SETSR'.format(zone)
@@ -500,6 +661,15 @@ def _format_set_treble(zone: int, treble: float) -> bytes:
     else:
        return 'Z{:0=2}TREB{:0=3}'.format(int(zone), int(treble))
 
+def _format_set_balance(zone: int, balance: float) -> bytes:
+    zonebalance[int(zone)] = balance
+    if balance > 0:
+       return 'Z{:0=2}BALR{:0=1}'.format(int(zone), int(balance))
+    elif balance < 0:
+       return 'Z{:0=2}BALL{:0=1}'.format(int(zone), abs(int(balance)))
+    else:
+       return 'Z{:0=2}BALC'.format(int(zone))
+
 def _format_set_source(zone: int, source: int) -> str:
     source = int(max(1, min(int(source), 6)))
     zonesrc[int(zone)] = source
@@ -515,12 +685,11 @@ def _format_all_off() -> str:
     return 'ALLOFF'.format()
 
 def all_off_keypad():
-    time.sleep(0.5) # Nuvo does NOT process commands immediately after ALLOFF
     zonesalloff = 1
-    for count in range(0,13):
+    for count in range(0,21):
         if zonepwr[int(count)] == 1:
             zonesalloff = 0
-    for count in range(0,13):
+    for count in range(0,21):
         if zonesalloff == 0:
             zonepwralloff[int(count)] = zonepwr[int(count)]
             zonepwr[int(count)] = 0
@@ -532,6 +701,14 @@ def all_off_keypad():
             callbackint = int(callbackobj)
             media_callback[callbackint]()
 
+def SimpleseZonePwrOff(match):
+    zone = abs(int(match[1]))
+    zonepwr[zone] = 0
+    if zone in media_zonetable:
+        callbackobj = media_zonetable.index(zone)
+        callbackint = int(callbackobj)
+        media_callback[callbackint]()
+
 def keypad_lock_recv(match):
     zone = abs(int(match[1]))
     lock = match[2]
@@ -582,28 +759,38 @@ def get_nuvo(port_url, baud, all_off_recall):
                 MODEL = 'Unknown'
             return MODEL
 
-        def _send_request(self, request):
+        def _send_request():
             """
             :param request: request that is sent to the nuvo
             :return: bool if transmit success
             """
-            #format and send output command
-            lineout = "*" + request + "\r"
-            _LOGGER.debug('Sending "%s"', request)
-            try:
-                port.write(lineout.encode())
-            except:
-                _LOGGER.error('Unexpected port error when sending command.')
-                return False
-            time.sleep(0.1) #Must wait in between commands for MPU to process
-            return True
+            global port
+            request = ''
+            # format and send output command
+            while request != 'EXIT':
+                request = cmd_queue.get(block=True)
+                lineout = "*" + request + "\r"
+                _LOGGER.debug('Sending "%s"', request)
+                try:
+                    port.write(lineout.encode())
+
+                    if request == 'ALLOFF':
+                        time.sleep(1) # Nuvo does NOT process commands immediately after ALLOFF
+                    if request[3:5] == 'ON':
+                        time.sleep(0.05) # Give Nuvo zone time to power up
+                    time.sleep(0.1)  # Must wait in between commands for MPU to process
+                except:
+                    _LOGGER.error('Unexpected port error when sending command.')
+
+        SendThread = threading.Thread(target=_send_request, args=(), daemon=True)
+        SendThread.start()
 
         def _process_request(self, request: str):
             """
             :param request: request that is sent to the nuvo
             """
             # Send command to device
-            self._send_request(request)
+            cmd_queue.put(request)
 
         def _listen():
             start_time = time.time()
@@ -697,7 +884,11 @@ def get_nuvo(port_url, baud, all_off_recall):
             """
             if zoneinit[int(zone)] == 0:
                try:
-                  rtn = ZoneStatus.from_string(self._process_request(
+                   if MODEL != 'CONCERTO':
+                       rtn = ZoneConnectStatus.from_string(self._process_request(
+                                    _format_zone_status_request(zone)))
+                   else:
+                       rtn = ZoneStatus.from_string(self._process_request(
                                     _format_zone_status_request(zone)))
                except:
                   rtn = None
@@ -729,7 +920,6 @@ def get_nuvo(port_url, baud, all_off_recall):
                         if zonepwr[int(zone)] != zonepwr[group_member]:
                             if zonepwr[int(zone)] == 1:
                                 self.set_power(group_member, True)
-                                time.sleep(0.05) # Give Nuvo zone time to power up
                             else:
                                 self.set_power(group_member, False)
                         if zonevol[int(zone)] != zonevol[group_member]:
@@ -754,8 +944,12 @@ def get_nuvo(port_url, baud, all_off_recall):
                             self.set_source(group_member, zonesrc[int(zone)])
                         zonecnt += 1
 
-            return ZoneStatus(int(zone), zonestrpwr,
-                zonesrc[int(zone)], 0, zonestrvol, False)
+            if MODEL == 'CONCERTO':
+                return ZoneStatus(int(zone), zonestrpwr,
+                    zonesrc[int(zone)], zonestrvol, False)
+            else:
+                return ZoneConnectStatus(int(zone), zonestrpwr,
+                    zonesrc[int(zone)], 0, zonestrvol, False)
 
 
         @synchronized
@@ -765,20 +959,30 @@ def get_nuvo(port_url, baud, all_off_recall):
             a response
             """
             if zonesetinit[int(zone)] == 0:
-               try:
-                  rtn = ZonesetStatus.from_string(
-                                    self._process_request(
-                                    _format_zoneset_status_request(zone)))
-               except:
-                  rtn = None
-               zonesetinit[int(zone)] += 1
-               return rtn
+                try:
+                    if MODEL != 'CONCERTO':
+                        rtn = ZonesetStatus.from_string(
+                                     self._process_request(
+                                     _format_zoneset_status_request(zone)))
+                    else:
+                        rtn = ConcertoZonesetStatus.from_string(
+                                     self._process_request(
+                                     _format_zoneset_status_request(zone)))
+                except:
+                    rtn = None
+                zonesetinit[int(zone)] += 1
+                return rtn
             else:
-               return ZonesetStatus(int(zone), zoneoverride[int(zone)],
+                if MODEL != 'CONCERTO':
+                    return ZonesetStatus(int(zone), zoneoverride[int(zone)],
                                     zonebass[int(zone)],
                                     zonetreble[int(zone)],
                                     zonegroupsrc[int(zone)],
                                     zonevolreset[int(zone)], False)
+                else:     
+                    return ConcertoZonesetStatus(int(zone), zonebass[int(zone)],
+                                    zonetreble[int(zone)],
+                                    zonebalance[int(zone)], 0, 0, 0, False)
 
         @synchronized
         def join_players(self, zone, group_members):
@@ -894,6 +1098,10 @@ def get_nuvo(port_url, baud, all_off_recall):
         def set_bass(self, zone: int, bass: float):
             self._process_request(_format_set_bass(zone, bass))
 
+        @synchronized
+        def set_balance(self, zone: int, balance: float):
+            self._process_request(_format_set_balance(zone, balance))
+
         @synchronized
         def set_source(self, zone: int, source: int):
             if zone == 0:
@@ -905,8 +1113,7 @@ def get_nuvo(port_url, baud, all_off_recall):
         @synchronized
         def page_off(self, page_source, page_zones):
             while page_active[1] == 1:
-                _LOGGER.info('Paging on service call currently in progress!  Waiting to turn off paging.')
-                time.sleep(1)
+                _LOGGER.info('Paging on service call currently in progress when paging off service call requested!  This should be ok, but could put zones in the incorrect state.')
             zonecnt = 0
             while zonecnt < len(page_zones):
                self.set_volume(int(page_zones[zonecnt]),
@@ -936,7 +1143,6 @@ def get_nuvo(port_url, baud, all_off_recall):
                    zonemutesave[int(page_zones[zonecnt])] = zonemute[int(page_zones[zonecnt])]
                    if zonepwr[int(page_zones[zonecnt])] == 0:
                        self.set_power(int(page_zones[zonecnt]), True)
-                       time.sleep(0.05)
                    if zonesrc[int(page_zones[zonecnt])] != page_source:
                        self.set_source(int(page_zones[zonecnt]), page_source)
                    self.set_volume(int(page_zones[zonecnt]), percent_to_db(int(page_volume[zonecnt])))
@@ -949,7 +1155,7 @@ def get_nuvo(port_url, baud, all_off_recall):
         @synchronized
         def mute_all(self):
             self._process_request(_format_mute_all())
-            for count in range(1,13):
+            for count in range(1,21):
                 zonemute[int(count)] = 1
                 if count in media_zonetable:
                     callbackobj = media_zonetable.index(count)
@@ -959,7 +1165,7 @@ def get_nuvo(port_url, baud, all_off_recall):
         @synchronized
         def unmute_all(self):
             self._process_request(_format_unmute_all())
-            for count in range(1,13):
+            for count in range(1,21):
                 zonemute[int(count)] = 0
                 if count in media_zonetable:
                     callbackobj = media_zonetable.index(count)
@@ -969,7 +1175,7 @@ def get_nuvo(port_url, baud, all_off_recall):
         @synchronized
         def all_off(self):
             self._process_request(_format_all_off())
-            for count in range(0,13):
+            for count in range(0,21):
                 zonepwralloff[int(count)] = 0
                 zonepwr[int(count)] = 0
                 if count in media_zonetable:
@@ -978,3 +1184,5 @@ def get_nuvo(port_url, baud, all_off_recall):
                     media_callback[callbackint]()
 
     return NuvoSync(port_url)
+
+
